#!/bin/ksh
# ident @(#)legato.ksh	1.55     96/10/09 SMI
#
# Copyright 1996 Sun Microsystems, Inc. All Rights Reserved

#
#  Library of NetWorker functions -- Oct 24, 1995.
#	

LEGATO_BIN=/usr/bin/nsr
LEGATO_SBIN=/usr/sbin/nsr
LEGATO_RES=/nsr/res/nsr.res
LIB_MESSAGE_LOG=/var/opt/netra/log

# A directory where Legato keeps some needed utilities:
DRVDIR=/etc/LGTOuscsi

# Some strings used by most of the modules
NO_DAEMON_BANNER="No backup daemons running."
NO_DAEMON_ERR="No backup daemons running.  They may be started through the <a href=\"$MODCGI/maintenance.cgi\">Backup Process Maintenance</a> page"

#
# initial policies
#
# The new Netra Single-Server License won't let us add our own NSR schedule
# "Netra NFS" under nsradmin -- sometimes it works, sometimes it complains.
# So instead, we're modifying Default, which should always exist.
#
#SCHEDULE_NAME="Netra NFS"
SCHEDULE_NAME="Default"
SCHEDULE="full incr incr incr incr incr incr"
PREAUTH_ENABLER="77d4f9-3bebbc-af6f2e"

log_msg()
{
	echo `date "+%D %T"`: "$*" >> $LIB_MESSAGE_LOG
}

initial_setup()
{
	# get our hostname
	server=`uname -n`

	# check for pre-authorized single tape functionality;
	# otherwise apply enabler
	fgrep $PREAUTH_ENABLER $LEGATO_RES > /dev/null 2>&1
	if [[ $? -ne 0 ]]; then
		msg=$( ${LEGATO_SBIN}/nsrcap -c $PREAUTH_ENABLER 2>&1 )
		ret=$?
		log_msg $msg
		if [ $ret != 0 ]; then
			return $ret
		fi
	fi

	# check for Netra NFS schedule; create otherwise
	fgrep "$SCHEDULE_NAME" $LEGATO_RES > /dev/null 2>&1
	if [[ $? -ne 0 ]]; then
		msg=`${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF
			create  type: NSR schedule; 
				name: ${SCHEDULE_NAME};
				period: Week;
				override:;
		EOF
		`
		ret=$?
		log_msg $msg
		if [ $ret != 0 ]; then
			return $ret
		fi
	fi

	# delete the automatically config'd non-compressed tape device;
	# don't really care about return value.
	msg=`${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF
		delete  type: NSR device
	EOF
	`
	log_msg $msg

        # initial policy configuration of the backup server
	msg=`${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF
		. type: NSR client; name: ${server}
		update  schedule: ${SCHEDULE_NAME};
			period: Week;
			action: ${SCHEDULE};
			override:;
			browse policy: Month;
			retention policy: Month;
			directive: Unix standard directives;
			aliases: root;
			administrator: root@${server}, setup
		. type: nsr
		update  administrator: root@${server}, setup
		. type: nsr group
		update  inactivity timeout: 1000;
        EOF
        `
	ret=$?
	log_msg $msg
	return $ret
}

get_daemon_list()
{
#	${LEGATO_SBIN}/nsr_shutdown -n
	ps -ef | grep nsr | grep -v grep
}

get_media_inventory()
{
	server=`uname -n`
	${LEGATO_SBIN}/nsrmm -s $server -C | grep -v "no such device"
}

get_jukebox_inventory()
{
	${LEGATO_SBIN}/nsrjb -C
}

#
# Is any long-term operation in progress?
# If so, print which one and return 0.
# If not, don't print anything, and return 1.
#
# Expects to have ps output passed in as $1.
#
operation_in_progress()
{
	which_op=""
	if [[ $which_op = "" && $1 = *ncheck* ]]; then
		which_op="Ncheck"
	fi
	if [[ $1 = *savegr* ]]; then
		which_op="Backup"
	fi
	if [[ $1 = *recover* || $1 = *mmrecov* || $1 = *scanner* ]]; then
		if [[ $1 = *recover* ]]; then
			crash=""
		else
			crash="Crash "
		fi
		if [[ $which_op != "" ]]; then
			which_op="${which_op} and ${crash}recovery"
		else
			which_op="${crash}recovery"
		fi
	fi
	if [[ $which_op != "" ]]; then
		print $which_op
		return 0
	fi
	return 1
}

change_email_list()
{
	#
	# Usage: change_email_list user1 user2 ...
	#
	users="$*"
	server=`uname -n`

	msg=`${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF
		. type: nsr notification; name: Registration
		update action: "/usr/ucb/mail -s \"registration status for ${server}\" ${users}"

		. type: nsr notification; name: savegroup completion
		update action: "/usr/ucb/mail -s \"Nightly backup report for ${server}\" ${users}"

		. type: nsr notification; name: tape mount request 3
		update action: "/usr/ucb/mail -s \"Tape request for ${server}\" ${users}"

		. type: nsr notification; name: index size
		update action: "/usr/ucb/mail -s \"Index size for ${server}\" ${users}"
	EOF
	`

	return $?
}

start_full_now()
{
	OUTPUT=/tmp/backup$$

	${LEGATO_SBIN}/savegrp -l Full Default >> $OUTPUT &
	msg="Default group started with level full"
	log_msg $msg
	return 0
}

recover_file_list()
{
	#
	# Usage: recover_file_list file [file ...]
	#
	# <file> should not be /
	# If the variable $WWW_date is set, it will be passed to
	# the recover command instead of the default "0 days ago".
	#
	# If $WWW_date is set, we'll restore as of that date.
	# If $WWW_relocate_dir is set, we'll relocate to there.
	#
	server=`uname -n`
	filelist="$*"

	# Set -d and -t options:
	if [[ $WWW_date != "" && $WWW_date != "0 days ago" ]]; then
		changetime="changetime $WWW_date"
	else
		changetime=""
	fi

	if [[ $WWW_relocate_dir != "" ]]; then
		relocate="relocate $WWW_relocate_dir"
	else
		relocate=""
	fi

	log_msg "Starting recovery of specified files"

	NSRHOME=/export/admin/nsr
	MMFILE=${NSRHOME}/mm/mmvolume
	IDXFILE=${NSRHOME}/index/${server}/db
	LOGFILE=/tmp/recover.$$

	# Run recover "interactively", as suggested by paul@jupiter.Legato.COM:
	# Note that msg is not used, and is always null.
	cd /
	${LEGATO_BIN}/recover -s ${server} -c ${server} -iN \
	    <<- EOF >$LOGFILE 2>&1
		$relocate
		$changetime
		add $filelist
		delete $MMFILE
		delete $IDXFILE
		recover
		quit
	EOF
	let ret=$?
	# Unfortunately, in interactive mode, the only way recover will
	# exit with nonzero status is for a usage message.  There's no
	# way for us to tell whether the recover was successful or not.

	# FIXME -- need to remove this logfile.  However, for debugging,
	# we'll want to keep it around, so for now, we're not removing it.
	#rm $LOGFILE
	log_msg "Finished recovery of specified files -- see $LOGFILE for details"

	return $ret
}

recover_filesystem()
{
	#
	# Usage: recover_filesystem <pathname> <days_ago>
	#
	# <pathname> should not be /
	# <days_ago> must be a number; should be zero for today
	#
	pathname=$1
	days_ago=$2
	server=`uname -n`

	log_msg "starting recovery of $pathname"
	
	msg=`${LEGATO_BIN}/recover -s ${server} -c ${server} -t "${days_ago} day ago" -a -iY ${pathname}`
	ret=$?

	log_msg $msg
	return $ret
}

recover_index_and_config()
{
	#
	# Usage: recover_index_and_config <devpath>
	#
	# The tape with the last bootstrap should be in the tapedrive
	#
	devpath=$1
	server=`uname -n`

	if [[ $devpath = Jukebox* ]]; then
		devpath=$( ${LEGATO_SBIN}/nsradmin -s $server -i - <<- EOF
			show "devices"
			print type: NSR jukebox
		EOF
		)
		devpath=$( print "$devpath" | head -1 | awk '{print $2}' \
			  | sed 's/;//g' )
	fi

	scanner_out=$( ${LEGATO_SBIN}/scanner -B $devpath 2>&1 )
	# scanner seems to work somehow even if it's pointed at the wrong
	# device compression, e.g. pointed at 0hbn when the tape is compressed.

	# Get the ssid, or appropriate error message
	if [[ $scanner_out = *"No bootstrap located"* ]]; then
		print "Couldn't find any index information on the tape"
		return 1
	fi

	ssid=$( print "$scanner_out" | awk '/Bootstrap/ { print $3 }' )
	if [[ $ssid = "" ]]; then
		print "Couldn't read Save Set ID from tape"
		return 1
	fi
	recover_index $ssid > /dev/null &
	return 0
}

stop_daemons()
{
	${LEGATO_SBIN}/nsr_shutdown -q
}

start_daemons()
{
	# Re-start the daemons:
	# ${LEGATO_SBIN}/nsrd > /dev/null &
	/etc/init.d/networker start

	# The networker script should sync the networker resources
	# with the sbu.cf file, so we no longer need to do it here.
}

recover_index()
{
	#
	# mmrecov calls various other nsr programs without bothering
	# to specify the paths to those binaries, so it requires that
	# we extend our path:
	#
	PATH=$PATH:/usr/sbin/nsr:/usr/bin/nsr:/usr/lib/nsr

	#
	# Note also that mmrecov uses the default device in the legato
	# resource file (even though its purpose is to recover the
	# resource file assuming that you don't have one).  So make
	# sure that the default device is set correctly (e.g. don't
	# let it default to uncompressed if the tape is compressed).
	#

	#
	# mmrecov requires SSID followed by 3 blank input lines
	#  -- DO NOT REMOVE
	#
	${LEGATO_SBIN}/mmrecov <<-EOF > /dev/null
$1
		


	EOF

	# mmrecov calls nsrindexasm, which starts a recover process
	# in the background then exits without giving us any way of
	# determining when the recover process is done.
	# We have to wait for the recover to finish, then stop
	# the daemons and restart them with the new resource files,
	# before proceeding with file recovery.  Apparently the
	# only way to do this (as per Legato) is to loop on ps.
	sleep 60
	while ( ps -ef | grep -v grep | grep -w recover >/dev/null 2>&1 )
	do
		sleep 10
	done

	stop_daemons

	# Copy the newly restored resource files on top of the old
	# possibly bogus ones:
	if [[ -d /nsr/res.R ]]; then
		rm -rf /nsr/res.old
		mv /nsr/res /nsr/res.bak
		mv /nsr/res.R /nsr/res
	fi

	start_daemons

	# For file recovery, we will call recover with the -iN option,
	# which means that it won't overwrite files already on the disk.
	# This is fine in most cases, but it means that if we had quotas
	# before the crash, the zero-length quota file which was created
	# at boot time by quotad will prevent recovery of the quotas file.
	# We can get around this by running rm'ing any zero-length
	# /export/quotas so that legato will recover the quotas file.
	# If there is already a zero-length quota file, then it probably
	# came from the configuration floppy, so leave it alone.
	quotafile=/export/quotas
	if [[ -a $quotafile && ! -s $quotafile ]]; then
		rm $quotafile
	fi

	#
	# ... Would be nice to write to a file somewhere saying that
	# recovery has finished ... then the status page will display
	# that info.  Though watching to see when mmrecov finishes
	# running may be nearly as good, and that's what the status
	# page does now.
	#

	return 0
}

get_res_attr()
{
	#
	# Output an attribute from resource of a certain type to stdout
	#
	res=$1
	attr=$2
	server=`uname -n`

	${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF | sed -e "s/^ *${attr}: *//" -e "s/;$//"
		show ${attr}
		print type: ${res}
	EOF
}

get_messages()
{
	#
	# Output server messages to stdout
	#
	get_res_attr "nsr" "message list"\
	| awk '
		/^",* *$/ {
			next;
		}
		/^[ 	]*$/ {
			next;
		}
		/\\$/ {
			saveline = saveline substr($0, 13, length-13);
			next;
		}
		saveline != "" {
			print saveline $0;
			saveline = "";
			next;
		}

		{
			print substr($0, 13, length);
			next;
		}

		END {
			if (saveline != "")
				print saveline;
		}
	'
}

get_session()
{
	#
	# Output server session info to stdout
	#
	# See comment under get_pending for explanation of nawk stuff
	get_res_attr "nsr" "session" 2>&1 | nawk '
		/\\$/ { gsub("\\\\", ""); printf("%s", $0); next; }
		      { print $0; }
	'
}

get_pending()
{
	#
	# Output pending status to stdout
	#
	# The Pending messages come from Legato with \\\n's in them,
	# which really look ugly, so get rid of them by telling nawk
	# to print without a newline any line that ends with a backslash:
	get_res_attr "nsr" "pending" 2>&1 | nawk '
		/\\$/ { gsub("\\\\", "");
			if ($0 != "") print $0; next; }
		      { if ($0 != "") print $0; }
	'
}

get_device_volume()
{
	#
	# Ouput the name of the volume (if any) mounted on device (to stdout)
	#
	get_res_attr "nsr device" "volume name"
}

get_device_message()
{
	#
	# Output the last activity message for device (to stdout)
	#
	get_res_attr "nsr device" "message"
}

get_device_name()
{
	#
	# Output the pathname of the device (to stdout)
	#
	get_res_attr "nsr device" "name"
}

get_index_info()
{
	#
	# Output a brief index report to stdout
	#
	${LEGATO_SBIN}/nsrls `uname -n`
}

remove_oldest_cycle()
{
	#
	# Remove the oldest cycle and output report to stdout
	#
	${LEGATO_SBIN}/nsrim -l -c `uname -n`
}

reset_logs()
{
	#
	# Trim (ie, delete) the log files
	#
	rm /nsr/logs/*
}

compress_index()
{
	#
	# Run an index check/compress (in background)
	#
	${LEGATO_SBIN}/nsrck -F `uname -n` &
}

# mount prints out a message about the label of the tape mounted.
# It also leaves auto media management ON when it's finished.
mount()
{
	#
	# Attempt to mount the tape in device
	#
	server=`uname -n`
	auto_media_management_off
	${LEGATO_SBIN}/nsrmm -s $server -m
	ret=$?
	auto_media_management_on
	return $ret
}

label_and_mount_force()
{
	#
	# Label and mount -- now!
	#
	# unmount will set $server and will leave amm off
	#
	unmount

	# For jukebox, can't label unless we've specified $WWW_jbunit.
	device=$( get_default_device )
	if [[ $device = Jukebox* ]]; then
		if [[ $WWW_jbunit = "" || $WWW_jbunit = "None" ]]; then
			print "Must specify jukebox slot"
			return 1
		fi
		# jbunit will look something like: 11: -* or 4: yeast.004
		# strip off only the slot number:
		WWW_jbunit=${WWW_jbunit%%:*}
		${LEGATO_SBIN}/nsrjb -L -Y -b Default -S $WWW_jbunit
		return $?

		# Jukebox can't load/mount in the same command as label:
		# to mount it afterward, add the following command
		# (but remember to save return val from prev. cmd first):
		#${LEGATO_SBIN}/nsrjb -l -S $WWW_jbunit
	fi


	${LEGATO_SBIN}/nsrmm -s $server -l -m -y -b Default 2>&1
	let ret=$?

	auto_media_management_on
	return $ret
}

#
# unmount and leave auto media management OFF
#
unmount()
{
	server=`uname -n`

	auto_media_management_off
	msg=$( ${LEGATO_SBIN}/nsrmm -s $server -u )
	let status=$?
	if [[ $status -ne 0 && msg = *"no such"* ]]; then
		${LEGATO_SBIN}/nsrjb -H
		let status=$?
	fi
	return $status
}

auto_media_management_off()
{
	${LEGATO_SBIN}/nsradmin -s $server -i - <<- EOF > /dev/null 2>&1
		. type: NSR device
		update auto media management: No
		. type: NSR jukebox
		update auto media management: No
	EOF
}

auto_media_management_on()
{
	${LEGATO_SBIN}/nsradmin -s $server -i - <<- EOF > /dev/null 2>&1
		. type: NSR device; enabled: Yes
		update auto media management: Yes
		. type: NSR jukebox
		update auto media management: Yes
	EOF
}

get_tape_report()
{
	#
	# Output a default tape report to stdout
	#
	${LEGATO_SBIN}/mminfo -m
}

#
# return all the legato devices, including disabled ones
#
get_legato_devices()
{
	#
	# First, get a list of the devices associated with jukeboxes:
	#
	jb_out=$( ${LEGATO_SBIN}/nsrjb -C 2>/dev/null )
	jbdevs=$( print "$jb_out" | grep drive \
		 | sed -e 's/.*(//' -e 's/).*//' )

	${LEGATO_SBIN}/nsrmm -C | while read line; do
		path=/dev/rmt/${line##*/dev/rmt/}
		path=${path%%, *}

		if [[ $line != *disabled ]]; then
			# Check whether it's associated with a jukebox:
			if [[ $jbdevs = *${path}* ]]; then
				print "Jukebox: $path"
				# FIXME also want name of jukebox
			else
				if [[ $line = "(nothing) "* ]]; then
					type=${line##*mounted on }
					type=${type%% tape*}
				else
					type=${line%% tape*}
				fi
				print "$path $type"
			fi
		fi
	done
}

#
# Next routine is obsolete and should be deleted
# (it's much easier to use nsrmm to get this info).
#
get_legato_devices_old()
{
	server=`uname -n`

	${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF \
		| sed -e "s/;$//" -e "s/: /:/" | awk '
    BEGIN {FS=":"}
    /media type/	{ media_type=$2; next }
    /message/		{ message=$2; next }
    /enabled/		{ enabled=$2; next }
    /volume name/	{ if ((enabled == "Yes") && (message == "disabled"))
				message = "";
			  print name,media_type,$2,message; next}
    /name/		{ name=$2; next } '
		show "name"
		show "media type"
		show "volume name"
		show "message"
		show "enabled"
		print type: nsr device
	EOF
}

#
# Show all the enabled devices currently in legato (should only be 1!).
# Returns a string such as: "/dev/rmt/0hbn 8mm 5GB".
# If there is no legato default device, it returns a null string.
#
get_default_device()
{
	server=`uname -n`

	${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF \
		| sed -e "s/;$//" -e "s/: /:/" | nawk '
    BEGIN {FS=":"}
    /devices/ { jbdev[++njbdev] = $0; next }
    /model/ { jbmodel[++njbmod] = $NF; next }
    /media type/ { media_type=$2; next }
    /name/ { name=$2;
	     for (i=1; i<=njbdev; ++i)
		if (index(jbdev[i], name) > 0)
		    { print "Jukebox", jbmodel[i]; exit }
	     print name,media_type; exit
	   }'
		show "devices"
		show "model"
		print type: nsr jukebox
		show "name"
		show "media type"
		print type: nsr device; enabled: Yes
	EOF
}

#
# Print the media type, as a string like "4mm" or "8mm".
# Pass in the pathname of the device as $1,
# and a line containing as much model info as we know in $2.
# We will try to mt stat the device first, since mt stat always
# gets the right answer, but if that fails (which it will if no
# tape is currently loaded) then we'll try to guess it from the
# model name
#
get_media_type_from_dev()
{
	dev=$1
	devline=$2

	# Try to get it from the device, if we have one:
	if [[ $dev != "" ]]; then
		statout=$( mt -f $dev stat 2>&1 | head -1 )
		if [[ $statout = *4mm* ]]; then
			print "4mm"
			return 0
		elif [[ $statout = *8mm* ]]; then
			print "8mm"
			return 0
		fi
		# Unfortunately, the DLT drive simply calls itself
		# "SCSI tape drive" in mt stat, so we can't detect it.
	fi

	# Else try to guess the media type from the device model:
	if [[ $devline = *Python* || $devline = *ARC* ]]; then
		print "4mm"
		return 0
	elif [[ $devline = *EXB* || $devline = *EXABYTE* ]]; then
		print "8mm"
		return 0
	elif [[ $devline = *DLT* ]]; then
		print "DLT"
		return 0
	elif [[ $devline = *"TZ Media Changer"* ]]; then
		print "DLT"
		return 0
	else
		# mt stat can't tell us, so just default it:
		return 1
	fi
}

#
# Crib notes on mt stat:
# - device is present, ready and working:
#   mt stat prints a buncha info and returns 0
# - device is present and working, but busy:
#   mt says "device busy" and I THINK returns 1
# - device present and working but no tape loaded:
#   mt says "no tape loaded or drive offline" -- what does it return?
# - device special file doesn't exist:
#   mt says "No such file or directory" and returns 1
# - device special file exists but device doesn't:
#   mt says "No such device or address" and returns 1
#
get_devices()
{
	inquire=` $DRVDIR/inquire | egrep "Tape|Jukebox" `
	changers=` $DRVDIR/changers -d `

	devices=""

	print "$inquire" | while read devline ; do
		scsidev=${devline%%:*}
		if [[ $changers != *$scsidev* ]]; then
			# scsidev is a not part of a jukebox, but is
			# a normal single tape, so add it to the list:
			thisdev=` print "${devline##* }" `
			# devices always come out as /dev/rmt/?mbn --
			# strip off the useless mbn part.
			# Might be more robust to strip off anything after
			# the last digit, if I could figure out a way to
			# persuade ksh to do that ...
			thisdev=${thisdev%%mbn}

			mediatype=$( get_media_type_from_dev $thisdev \
							     "$devline" )

			devices="$thisdev $mediatype\n$devices"
		fi
	done

	# Now list the jukeboxes:
	if [[ $changers != "" ]]; then
	    let i=0
	    print "$changers" | while read devline ; do
	# parse a "changers" line like:
	# scsidev@0.0.0:Vendor <EXABYTE>, Product <EXB-210>, Revision <3.01>
		i=i+1
		vendor=${devline##*Vendor \<}
		vendor=${vendor%%\>*}
		product=${devline##*Product \<}
		product=${product%%\>*}

		# read to the next blank line, trapping lines
		# starting with "Data Transfer Element":
		while read line ; do
			if [[ $line = "" ]]; then
				break;
			elif [[ $line = *"Data Transfer Element"* ]]; then
				dte=${line##* }
				dte=` print "$inquire" | grep $dte \
				     | awk '{print $NF;}' `
				# remove from the single-tape list:
				if [[ $devices != "" ]]; then
					devices=` print $devices \
						 | grep -v $dte `
					devices="$devices\n"
				fi
			fi
		done
		if [[ line = "" ]]; then
			break;
		fi

		#devices="${devices}Jukebox: $vendor $product\n"
		devices="${devices}Jukebox $product\n"
	    done
	fi

	print -n "$devices"
}

#
# Add a jukebox to Legato, using gross jb_config since we can't
# get the device type string from "changers -d" that "savegrp" expects.
# This routine assumes that $server is already set.
# $1 is the string "Jukebox" and can be ignored;
# $2 is the type, e.g. "EXB-210" or "TZ Media Changer".
# Device will be added enabled;
# all other devices and jukeboxes will be removed.
#
add_legato_jukebox()
{
	devtype=$2

	#
	# Remove all existing devices and jukeboxes
	#
	unmount
	${LEGATO_SBIN}/nsradmin -s $server -i - <<- EOF
		delete type: NSR jukebox
		. type: NSR device
		update auto media management: No
		delete type: NSR device
	EOF

	# Try to get the media type:
	mediatype=$( get_media_type_from_dev "" "$devtype")
	if [[ $mediatype = 4mm ]]; then
		typeletter=c
	elif [[ $mediatype = 8mm ]]; then
		typeletter=d
	elif [[ $mediatype = DLT ]]; then
		typeletter=g
	else
		# pick random default of 4mm
		typeletter=c
	fi

	# Figure out which number jukebox this is, i.e. which line
	# of changers' output referenced this jb type:
	DRVDIR=/etc/LGTOuscsi
	let i=0
	let jbnum=0
	dtestr=""
	${DRVDIR}/changers -d | while read line ; do
		if [[ $line = scsidev@* ]]; then
			if [[ $jbnum -ne 0 ]]; then
				break
			fi
			let i=$i+1
			if [[ $line = *$devtype* ]]; then
				let jbnum=$i
			fi
		elif [[ $jbnum -ne 0 \
			   && $line = *"Data Transfer Element"* ]]; then
			dtestr="${dtestr}
${typeletter}
"
		fi
	done
	# Now jbnum should be which jukebox this is on the system, and
	# dtestr contains the dte lines to be passed to jb_config.
	# If we want to spruce up the error testing, we can test
	# $jbnum -- if it's zero we didn't see the device we wanted.

	hostname=`uname -n`
	jbname=$hostname-jb$jbnum

	# Configure the jukebox and its device using jb_config.
	# Yes, all these blank lines are really gross, but apparently
	# it's the only way to get a jukebox configured reliably.
	# dtestr contains a "\nX\n" for every data transfer element
	# in this jukebox -- note that since it already contains a
	# trailing newline, we don't hit another one before the "no"
	# following it (which is the answer to whether we have barcode).
	${LEGATO_SBIN}/jb_config <<- EOF >/dev/null 2>&1

		$jbnum
		$jbname

		no
		${dtestr}no
	EOF

	# and turn auto media management on for the jukebox:
	${LEGATO_SBIN}/nsradmin -s $server -i - <<- EOF >/dev/null 2>&1
		. type: NSR jukebox; name: $jbname
		update auto media management: Yes
	EOF

	# We also may need to initialize the jukebox to a known state
	# in order to get auto media management to work with it.
	if [[ $devtype = \(*\) ]]; then
		${LEGATO_SBIN}/nsrjb -H -E
	fi
}

#
# Put a device into Legato's resource file.
# Device will be added enabled;
# all other devices and jukeboxes will be removed.
# Use this routine for jukeboxes as well as single tapes
# (passing lines in the format returned by get_devices).
# This routine assumes that $server is already set.
#
add_legato_device()
{
	dev=$1
	type=$2

	unmount
	${LEGATO_SBIN}/nsradmin -s $server -i - <<- EOF
		delete type: NSR jukebox
		. type: NSR device
		update auto media management: No
		delete type: NSR device
		create type: NSR device; name: $dev; media type: $type; \
			auto media management: Yes; enabled: Yes; \
			volume pool: Default
	EOF
	# FIXME -- really should do something about error indication
}

change_device()
{
	#
	# usage: change_device <pathname> <devtype>
	# If no device, pass "None" or "" as <pathname>.
	#
	# Single tape e.g.: name=/dev/rmt/0cbn, type=4mm
	# Jukebox e.g.: name="Jukebox 1", type="EXABYTE EXB-210"
	# Actually, devpath is more likely to be just "Jukebox".
	#
	devpath=$1
	devtype=$2
	server=`uname -n`

	# get the name legato will be using:
	if [[ $devpath = Jukebox* ]]; then
		jbnum=${devpath#* }
		devname="$server-jb$jbnum"
	else
		jbnum=""
		devname=$devpath
	fi

	# Find out whether this device is already in legato,
	# and whether it's already in legato's resource files:
	legdev=`get_default_device | grep "$devname" | grep "$devtype" `
	if [[ $legdev = "" ]]; then
		# We need to put the device into Legato before we enable it
		if [[ $devpath != Jukebox* ]]; then
			add_legato_device "$devname" "$devtype"
			ret=$?
		else
			add_legato_jukebox "$devname" "$devtype"
			ret=$?
		fi
	else
		ret=$?
	fi

	# FIXME -- may want to log success or failure here,
	# along with nsradmin output stored in $msg.
	return $ret
}

# If it's a jukebox, then we can only get a label if there's
# already a tape loaded.  So see if something is loaded/mounted,
# and if so, set $device to that for get_label to work with;
# otherwise, return error.
get_label_from_jukebox()
{
	jbout=$( ${LEGATO_SBIN}/nsrjb -C 2>/dev/null | grep "(mounted)" )
	if [[ $jbout = "" ]]; then
		LABEL=""
		print "No tape loaded in jukebox"
		return 1
	fi
	LABEL=${jbout%% \(mounted\)}
	LABEL=${LABEL##* }
	print $LABEL
}

get_label()
{
	server=`uname -n`
	device=`get_default_device | awk '{print $1;}'`
	if [[ $device = "" ]]; then
		print "Unable to get label: error: no tape device found"
		return 1
	fi

	if [[ $device = Jukebox* ]]; then
		get_label_from_jukebox
		return $?
	fi

	STAT=`mt -f $device stat 2>&1`
        if [[ $STAT =  *offline* || $STAT = *busy* ]]
	then
        	echo $STAT
		return 1
	fi

	# mount tape if not already mounted
       	LABEL=`${LEGATO_SBIN}/nsrmm -s $server -C 2>&1 | grep "$device"`

	# This will look like either:
	# 4mm tape yeast.003 mounted on /dev/rmt/0cbn, write enabled
	# 8mm 5GB tape yeast.003 mounted on /dev/rmt/0cbn, write enabled
	# (nothing) mounted on 8mm tape /dev/rmt/2mbn, device disabled
	# Note that we can't assume that the label is the third word,
	# so instead we assume that it's the first word following "tape".

       	if [[ $LABEL != *nothing* ]]
	then
		LABEL=${LABEL#*tape }
		LABEL=${LABEL%% *}
		print "$LABEL"
		return 0
	fi
	# If we get here, then it said nothing was mounted.

	# Try to mount:
	LABEL=`mount 2>&1`
	if [[ $LABEL = *error* ]]
	then
		# Filter out two disturbing and unuseful errors which
		# Legato is likely to spit out for new tapes:
		if [[ $LABEL = *"monitor device"* || $LABEL = *"I/O error"* ]]
		then
			LABEL="no label"
		fi
		print "Unable to get label from tape: $LABEL"
		return 1
	else
		echo $LABEL | awk '{ for (i = 0; i < NF; i++)\
			if ($i == "tape") print $(i+1) }'
		return 0
	fi
}

get_notifications()
{
        #
        # Output notification list to stdout
        # Note! assuming all notification instances are the same
        #
        get_res_attr "NSR notification; name: savegroup completion" "action" |\
	sed -e 's/.*\\"//' -e 's/[ \\]//' -e 's/\"//'
}

schedule_backup()
{
	time=$1 
	full_day=$2 
	server=`uname -n`
	client=$server

	#
	# create the schedule
	#	action: \"$schedule\";
	schedule=""
	for i in Sunday Monday Tuesday Wednesday Thursday Friday Saturday; do
        	if [[ $full_day == $i ]]; then
                	schedule="$schedule full"
        	else
                	schedule="$schedule incr"
        	fi
	done
	# but now we have a leading space, so delete it:
	schedule=${schedule# }

	#
	# enable autostart, update the start time and schedule
	#
	msg=`${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF
		. type: NSR group; name: Default
		update	autostart: Enabled;
			start time: "${time}:00"
	    	. type: NSR schedule; name: ${SCHEDULE_NAME}
		update	action: "$schedule";
	EOF
	`
	ret=$?
	if [ $ret != 0 ]; then
		log_msg $msg
	fi
	return $ret
}

unschedule_backup()
{
	server=`uname -n`

	#
	# disable autostart
	#
	msg=`${LEGATO_SBIN}/nsradmin -s $server -i - <<-EOF
		. type: NSR group; name: Default
		update autostart: Disabled;
	EOF
	`
	ret=$?
	if [ $ret != 0 ]; then
		log_msg $msg
		return $ret
	fi
}

# $index_duration is either a string containing the name of
# the browse policy, or else a number specifying the
# number of months to keep the index file.  If we've specified a number
# of months rather than an existing policy, then create a new policy.
set_index_duration()
{
	index_duration=$1
	server=`uname -n`
	client=$server

	netra_is_int $index_duration
	if [[ $? -eq 0 ]]; then
        	nsr_output=$( ${LEGATO_SBIN}/nsradmin -s $server -i - << EOF
                	. type:NSR policy; name:Netra_custom
                	delete
                	create type:NSR policy; name:Netra_custom;
                       	period:Months; number of periods:$index_duration
                	quit
        	EOF )
        	index_duration=Netra_custom
	else
		# strip off the double quotes
		index_duration=$( print "$index_duration" \
				 | sed 's/^\"//' \
				 | sed 's/\".*//' )
	fi

	# Now set the client's browse policy to the specified policy:
	nsr_output=$( ${LEGATO_SBIN}/nsradmin -s $server -i - <<EOF
        	. type:NSR client; name:$client
        	update browse policy: $index_duration
        	quit
	EOF )
}

# Make a string passable through the CGI mechanism
txt2cgi() {
	sed 's/;/\&\#59;/g' \
		| sed 's/+/\&\#43;/g' \
		| sed 's=/=\&\#47;=g' \
		| sed 's/\"/\&quot;/g' \
		| sed 's/ /+/g'
}


